# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django


class Migration(migrations.Migration):

    if django.VERSION < (1, 8, 0):
        dependencies = [
            ('auth', '__first__'),
        ]
    else:
        dependencies = [
            ('auth', '0006_require_contenttypes_0002'),
        ]
    if django.VERSION < (2, 0, 0):
        operations = [
            migrations.CreateModel(
                name='Background',
                fields=[
                    ('user', models.OneToOneField(related_name='initial_avatars', primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                    ('R', models.IntegerField()),
                    ('G', models.IntegerField()),
                    ('B', models.IntegerField()),
                ],
            ),
        ]
    else:
        operations = [
            migrations.CreateModel(
                name='Background',
                fields=[
                    ('user', models.OneToOneField(related_name='initial_avatars', primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
                    ('R', models.IntegerField()),
                    ('G', models.IntegerField()),
                    ('B', models.IntegerField()),
                ],
            ),
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.contrib.auth.models import AnonymousUser, User
from django.test import TestCase
from django.core.files.storage import default_storage
from django.template import Context, Template
from PIL import Image, ImageDraw
from initial_avatars.generator import AvatarGenerator
from initial_avatars.utils import AvatarShapeException
from datetime import datetime


class TestAvatarGenerator(TestCase):

    TEMPLATE = Template("{% load initialavatar %} {% get_initial_avatar user %}")

    def setUp(self):
        self.userA = User.objects.create_user(
            username='Stéphane',
            email='admin@axiome.io',
            password='top_secret'
        )
        self.genA = AvatarGenerator(
            self.userA,
            80
        )
        self.userB = User.objects.create_user(
            username='matt',
            first_name='matt',
            last_name='something',
            email='matt@automattic.com',
            password='top_secret'
        )
        self.genB = AvatarGenerator(
            self.userB,
            80
        )
        self.userC = User.objects.create_user(
            username='carlotta',
            first_name='carlotta',
            last_name='da Silva',
            email='cs@automattic.com',
            password='top_secret'
        )
        self.genC = AvatarGenerator(
            self.userC,
            80
        )
    def test_text(self):
        self.assertEqual(
            self.genA.text(),
            'S'
        )
        self.assertEqual(
            self.genB.text(),
            'MS'
        )
        self.assertEqual(
            self.genC.text(),
            'CS'
        )

    def test_font_size(self):
        self.assertEqual(
            self.genA.font_size(),
            720
        )
        self.assertEqual(
            self.genB.font_size(),
            640
        )

    def test_brightness(self):
        self.assertEqual(
            int(self.genA.brightness()),
            219
        )
        self.assertEqual(
            int(self.genB.brightness()),
            200
        )

    def test_background(self):
        self.assertEqual(
            self.genA.background(),
            (3, 254, 229)
        )
        self.assertEqual(
            self.genB.background(),
            (208, 207, 63)
        )

    def test_foreground(self):
        self.assertEqual(
            self.genA.foreground(),
            (0, 0, 0)
        )
        self.assertEqual(
            self.genB.foreground(),
            (0, 0, 0)
        )

    def test_name(self):
        self.assertEqual(
            self.genA.name(),
            "80x80_square.jpg"
        )
        self.assertEqual(
            self.genB.name(),
            "80x80_square.jpg"
        )

    def test_path(self):
        self.assertEqual(
            self.genA.path(),
            "avatars\\8ac609169cd18386d2e1800b253511ce\\80x80_square.jpg"
        )
        self.assertEqual(
            self.genB.path(),
            "avatars\\a777f9544392312fd067b7d4762ea083\\80x80_square.jpg"
        )

    def test_get_avatar_url(self):
        self.assertEqual(
            self.genA.get_avatar_url(),
            "http://django-initial-avatars.py/avatars/8ac609169cd18386d2e1800b253511ce/80x80_square.jpg"
        )
        self.assertIn(
            self.genB.get_avatar_url(),
            [
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;r=g&amp;d=mm",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;d=mm&amp;r=g",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=80&amp;d=mm",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=80",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=80&amp;r=g",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=80"
            ]
        )

    def test_get_avatar(self):
        default_storage.delete('avatars/8ac609169cd18386d2e1800b253511ce/80x80_square@2x.jpg')
        default_storage.delete(self.genA.path())

        self.assertFalse(default_storage.exists(self.genA.path()))

        self.assertEqual(
            self.genA.get_avatar(),
            '<img class="initial-avatar" src="http://django-initial-avatars.py/avatars/8ac609169cd18386d2e1800b253511ce/80x80_square.jpg" width="80" height="80"/>'
        )
        self.assertIn(
            self.genB.get_avatar(),
            [
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;r=g&amp;d=mm" width="80" height="80"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;d=mm&amp;r=g" width="80" height="80"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=80&amp;d=mm" width="80" height="80"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=80" width="80" height="80"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=80&amp;r=g" width="80" height="80"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=80" width="80" height="80"/>'
            ]
        )
        self.assertTrue(default_storage.exists(self.genA.path()))

    def test_last_modified(self):
        self.assertIsInstance(
            self.genA.last_modification(),
            datetime
        )
        self.assertIsInstance(
            self.genB.last_modification(),
            datetime
        )

    def test_template_tags(self):
        renderedA = self.TEMPLATE.render(Context({'user': self.userA}))
        renderedB = self.TEMPLATE.render(Context({'user': self.userB}))
        renderedAnon = self.TEMPLATE.render(Context({'user': AnonymousUser()}))

        self.assertEqual(
            renderedA,
            u' <img class="initial-avatar" src="http://django-initial-avatars.py/avatars/8ac609169cd18386d2e1800b253511ce/80x80_square.jpg" width="80" height="80"/>'
        )
        self.assertIn(
            renderedB,
            [
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;r=g&amp;d=mm" width="80" height="80"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=80&amp;d=mm&amp;r=g" width="80" height="80"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=80&amp;d=mm" width="80" height="80"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=80" width="80" height="80"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=80&amp;r=g" width="80" height="80"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=80" width="80" height="80"/>'
            ]
        )
        self.assertEqual(
            renderedAnon,
            u' <img src="" width="80" height="80"/>'
        )

    def test_view(self):
        responseA = self.client.get('/1/')
        responseB = self.client.get('/2/')
        responseAnon = self.client.get('/anon/')

        self.assertEqual(
            responseA.status_code,
            302
        )
        self.assertEqual(
            responseB.status_code,
            302
        )
        self.assertEqual(
            responseAnon.status_code,
            404
        )


class TestAvatarGeneratorNotDefault(TestCase):

    TEMPLATE = Template("{% load initialavatar %} {% get_initial_avatar user 150 'circle' %}")

    def setUp(self):
        self.userA = User.objects.create_user(
            username='JAB',
            email='admin@axiome.io',
            password='top_secret'
        )
        self.genA = AvatarGenerator(self.userA, size=150, shape='circle')
        self.userB = User.objects.create_user(
            username='matt',
            first_name='matt',
            last_name='something',
            email='matt@automattic.com',
            password='top_secret'
        )
        self.genB = AvatarGenerator(self.userB, size=150)

    def test_text(self):
        self.assertEqual(
            self.genA.text(),
            'J'
        )
        self.assertEqual(
            self.genB.text(),
            'MS'
        )

    def test_font_size(self):
        self.assertEqual(
            self.genA.font_size(),
            1350
        )
        self.assertEqual(
            self.genB.font_size(),
            1200
        )

    def test_brightness(self):
        self.assertEqual(
            int(self.genA.brightness()),
            222
        )
        self.assertEqual(
            int(self.genB.brightness()),
            200
        )

    def test_background(self):
        self.assertEqual(
            self.genA.background(),
            (157, 242, 216)
        )
        self.assertEqual(
            self.genB.background(),
            (208, 207, 63)
        )

    def test_foreground(self):
        self.assertEqual(
            self.genA.foreground(),
            (0, 0, 0)
        )
        self.assertEqual(
            self.genB.foreground(),
            (0, 0, 0)
        )

    def test_name(self):
        self.assertEqual(
            self.genA.name(),
            "150x150_circle.png"
        )
        self.assertEqual(
            self.genB.name(),
            "150x150_square.jpg"
        )

    def test_path(self):
        self.assertEqual(
            self.genA.path(),
            "avatars\\963f42f411a171630625f6163ef581c3\\150x150_circle.png"
        )
        self.assertEqual(
            self.genB.path(),
            "avatars\\a777f9544392312fd067b7d4762ea083\\150x150_square.jpg"
        )

    def test_get_avatar_url(self):
        self.assertEqual(
            self.genA.get_avatar_url(),
            "http://django-initial-avatars.py/avatars/963f42f411a171630625f6163ef581c3/150x150_circle.png"
        )
        self.assertIn(
            self.genB.get_avatar_url(),
            [
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;r=g&amp;d=mm",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;d=mm&amp;r=g",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=150&amp;r=g",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=150",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=150&amp;d=mm",
                "https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=150",
            ]
        )

    def test_get_avatar(self):
        default_storage.delete(self.genA.path())
        default_storage.delete('avatars/963f42f411a171630625f6163ef581c3/150x150_circle@2x.png')
        self.assertFalse(default_storage.exists(self.genA.path()))

        self.assertEqual(
            self.genA.get_avatar(),
            '<img class="initial-avatar" src="http://django-initial-avatars.py/avatars/963f42f411a171630625f6163ef581c3/150x150_circle.png" width="150" height="150"/>'
        )
        self.assertIn(
            self.genB.get_avatar(),
            [
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;r=g&amp;d=mm" width="150" height="150"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;d=mm&amp;r=g" width="150" height="150"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=150&amp;d=mm" width="150" height="150"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=150" width="150" height="150"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=150&amp;r=g" width="150" height="150"/>',
                '<img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=150" width="150" height="150"/>'
            ]
        )
        self.assertTrue(default_storage.exists(self.genA.path()))

    def test_last_modified(self):
        self.assertIsInstance(
            self.genA.last_modification(),
            datetime
        )
        self.assertIsInstance(
            self.genB.last_modification(),
            datetime
        )

    def test_template_tags(self):
        renderedA = self.TEMPLATE.render(Context({'user': self.userA}))
        renderedB = self.TEMPLATE.render(Context({'user': self.userB}))
        renderedAnon = self.TEMPLATE.render(Context({'user': AnonymousUser()}))

        self.assertEqual(
            renderedA,
            u' <img class="initial-avatar" src="http://django-initial-avatars.py/avatars/963f42f411a171630625f6163ef581c3/150x150_circle.png" width="150" height="150"/>'
        )
        self.assertIn(
            renderedB,
            [
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;r=g&amp;d=mm" width="150" height="150"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?s=150&amp;d=mm&amp;r=g" width="150" height="150"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;s=150&amp;d=mm" width="150" height="150"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?r=g&amp;d=mm&amp;s=150" width="150" height="150"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;s=150&amp;r=g" width="150" height="150"/>',
                u' <img class="gravatar" src="https://secure.gravatar.com/avatar/c0ccdd53794779bcc07fcae7b79c4d80.jpg?d=mm&amp;r=g&amp;s=150" width="150" height="150"/>',
            ]
        )
        self.assertEqual(
            renderedAnon,
            u' <img src="" width="150" height="150"/>'
        )

    def test_non_default_template_tags(self):
        TEMPLATE = Template("{% load initialavatar %} {% get_initial_avatar user 200 'square' %}")
        renderedA = TEMPLATE.render(Context({'user': self.userA}))

        self.assertEqual(
            renderedA,
            u' <img class="initial-avatar" src="http://django-initial-avatars.py/avatars/963f42f411a171630625f6163ef581c3/200x200_square.jpg" width="200" height="200"/>'
        )

    def test_view(self):
        responseA = self.client.get('/1/150/')
        responseB = self.client.get('/2/150/')
        responseAnon = self.client.get('/3/150/')

        self.assertEqual(
            responseA.status_code,
            302
        )
        self.assertEqual(
            responseB.status_code,
            302
        )
        self.assertEqual(
            responseAnon.status_code,
            404
        )


class TestAvatarGeneratorBadSettings(TestCase):

    def setUp(self):
        self.userA = User.objects.create_user(
            username='JAB',
            email='admin@axiome.io',
            password='top_secret'
        )

    def test_exception(self):
        self.assertRaises(
            AvatarShapeException,
            AvatarGenerator, self.userA, size=150, shape='triangle'
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404
from django.http import HttpResponse, HttpResponseRedirect
from django.views.decorators.http import last_modified
from datetime import date, timedelta
from .generator import GRAVATAR_DEFAULT_SIZE
from .utils import get_avatar_backend


def last_modified_func(request, id, size=GRAVATAR_DEFAULT_SIZE):
    try:
        u = User.objects.get(id=id)
    except User.DoesNotExist:
        return None
    avatar_backend = get_avatar_backend()
    return avatar_backend(u, int(size)).last_modification()


def avatar(request, id, size=GRAVATAR_DEFAULT_SIZE):
    user = get_object_or_404(User, id=id)
    avatar_backend = get_avatar_backend()
    url = avatar_backend(user, size=int(size)).get_avatar_url()
    try:
        response = HttpResponseRedirect(url)
        response['Cache-Control'] = 'max-age=2592000'
        response['Expires'] = (date.today() + timedelta(days=31)).strftime('%a, %d %b %Y 20:00:00 GMT')
        return response
    except Exception:
        return HttpResponse('Not Found', status=404)



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings


class Background(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        primary_key=True,
        related_name='initial_avatars',
        on_delete=models.CASCADE
    )
    R = models.IntegerField()
    G = models.IntegerField()
    B = models.IntegerField()



from __future__ import unicode_literals
import os

SECRET_KEY = 'fake-key'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'TEST_NAME': os.path.join(os.path.dirname(__file__), 'django_initial_avatars.sqlite'),
    }
}
INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'initial_avatars',
    'django_gravatar',
]
MEDIA_URL = 'http://django-initial-avatars.py/'
MEDIA_ROOT = 'tests/static'
ROOT_URLCONF = 'initial_avatars.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.urls import path
from .views import avatar

urlpatterns = [
    path('<int:id>/', avatar, name='avatar'),
    path('<int:id>/<int:size>/', avatar, name='avatar'),
]



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import unicode_literals
import os
import re
try:
    from django_gravatar.helpers import get_gravatar_url, has_gravatar
except ImportError:
    pass
from django.utils.html import escape
from django.utils import timezone
from django.conf import settings
from django.core.files.storage import default_storage, storages
from django.core.files.uploadedfile import InMemoryUploadedFile
from PIL import Image, ImageDraw, ImageFont
from io import BytesIO
from math import sqrt
from hashlib import md5
from datetime import datetime
from random import choice
from .utils import AVATAR_SHAPE_SETTINGS, AVATAR_FOREGROUND_COLORS, AvatarShapeException, AvatarForegroundColorException
from .models import Background
from .compat import urlopen

GRAVATAR_DEFAULT_SIZE = getattr(settings, 'GRAVATAR_DEFAULT_SIZE', 80)
AVATAR_SHAPE = getattr(settings, 'AVATAR_DEFAULT_SHAPE', 'square')
AVATAR_STORAGE_FOLDER = getattr(settings, 'AVATAR_STORAGE_FOLDER', 'avatars')
AVATAR_HIGH_RESOLUTION = getattr(settings, 'AVATAR_HIGH_RESOLUTION', False)
AVATAR_COLORS = getattr(settings, 'AVATAR_COLORS', False)

try:
    AVATAR_DEFAULT_FOREGROUND = AVATAR_FOREGROUND_COLORS[settings.AVATAR_DEFAUL_TEXT_COLOR]
except KeyError:
    raise AvatarForegroundColorException
except AttributeError:
    pass

try:
    AVATAR_STORAGE_BACKEND = storages[settings.AVATAR_STORAGE_BACKEND]
except AttributeError:
    AVATAR_STORAGE_BACKEND = default_storage


class AvatarGenerator(object):
    """
        inspired by https://github.com/4teamwork/ftw.avatar
    """

    def __init__(self, user, size=GRAVATAR_DEFAULT_SIZE, shape=AVATAR_SHAPE):
        self.user = user
        self.work_size = size * 10
        self.size = size
        self.shape = shape
        try:
            self.image_format = AVATAR_SHAPE_SETTINGS[shape]['image_format']
            self.content_type = AVATAR_SHAPE_SETTINGS[shape]['content_type']
            self.color_channels = AVATAR_SHAPE_SETTINGS[shape]['color_channels']
        except KeyError:
            raise AvatarShapeException
        self.css_class = None
        self.url = None
        if AVATAR_HIGH_RESOLUTION:
            self.high_res_url = None

    def name(self, high_res=False):
        """
            returns the name of the img file
        """
        if high_res:
            name = "{0}x{0}_{1}@2x.{2}".format(self.size, self.shape, self.image_format)
        else:
            name = '{0}x{0}_{1}.{2}'.format(self.size, self.shape, self.image_format)
        return name

    def path(self, high_res=False):
        """
            returns the path of the img file
        """
        user_hash = md5(os.path.join(self.user.username, self.user.first_name, self.user.last_name).encode('utf-8')).hexdigest()
        user_path = os.path.join(AVATAR_STORAGE_FOLDER, user_hash, self.name(high_res))
        return user_path

    def font_size(self):
        """
            returns the size of the font calculated according to the size of requested avatar
        """
        font_size = int(self.work_size * (1 - 0.1 * len(self.text())))
        return font_size

    def font(self):
        """
            returns an ImageFont object with the font used to generate the avatar
        """
        font_path = os.path.join(os.path.dirname(__file__), 'font', 'UbuntuMono-B.ttf')
        font_size = self.font_size()
        return ImageFont.truetype(font_path, size=font_size)

    def choose_random_background(self):
        try:
            backgroundObject = Background.objects.get(user=self.user)
            background = (backgroundObject.R, backgroundObject.G, backgroundObject.B)
        except Background.DoesNotExist:
            background = choice(AVATAR_COLORS)
            Background.objects.create(user=self.user, R=background[0], G=background[1], B=background[2])
        return background

    def background(self):
        """
            returns the background color based on the username md5
        """
        if AVATAR_COLORS:
            background = self.choose_random_background()
        else:
            hash = md5(self.user.username.encode('utf-8')).hexdigest()
            hash_values = (hash[:8], hash[8:16], hash[16:24])
            background = tuple(int(value, 16) % 256 for value in hash_values)
        return background

    def brightness(self):
        """
            returns the brightness of the background color
            explanation of the formula on http://www.nbdtech.com/Blog/archive/2008/04/27/Calculating-the-Perceived-Brightness-of-a-Color.aspx
        """
        rCoef = 0.241
        gCoef = 0.691
        bCoef = 0.068
        background = self.background()
        brightness = sqrt(rCoef * background[0]**2 + gCoef * background[1]**2 + bCoef * background[2]**2)
        return brightness

    def foreground(self):
        """
            returns black or white according to the brightness
        """
        try:
            return AVATAR_DEFAULT_FOREGROUND
        except NameError:
            brightness = self.brightness()
            if brightness > 130:
                return (0, 0, 0)
            else:
                return (255, 255, 255)

    def text(self):
        """
            returns the text to be printed on the avatar
            first letter of first_name and last_name if they exists
            first letter of the username if not

            Attempts to find first capital letter of last name. This supports
            names such as "Alexandre da Rocha" by using the correct form "AR".
        """
        if self.user.first_name and self.user.last_name:
            initial = self.user.first_name[:1].upper()

            match = re.search(r'[A-Z]', self.user.last_name)
            if match:
                # Use first capital letter
                initial += match.group()
            else:
                # No capitals found; just use first letter
                initial += self.user.last_name[:1].upper()
        else:
            initial = self.user.username[:1].upper()
        return initial

    def last_modification(self):
        """
            returns the avatar last_modification
        """
        try:
            if has_gravatar(self.user.email):
                info = urlopen(get_gravatar_url(email=self.user.email, size=self.size)).info()
                return datetime.strptime(info['Last-Modified'], "%a, %d %b %Y %H:%M:%S GMT")
        except NameError:
            pass
        if AVATAR_STORAGE_BACKEND.exists(self.path()):
            try:
                return AVATAR_STORAGE_BACKEND.modified_time(self.path())
            except AttributeError:
                return timezone.now()
        else:
            return None

    def genavatar(self):
        if self.shape == 'square':
            return self.gen_image_avatar(self.background())
        elif self.shape == 'circle':
            return self.gen_image_avatar((255, 0, 0, 0))
        else:
            raise AvatarShapeException

    def gen_image_avatar(self, background):
        work_image = Image.new(self.color_channels, (self.work_size, self.work_size), background)
        draw = ImageDraw.Draw(work_image)
        if self.shape == 'circle':
            draw.ellipse((0, 0, self.work_size, self.work_size), fill=self.background())
        center = self.work_size / 2
        draw.text((center, center), self.text(), fill=self.foreground(), font=self.font(), anchor = "mm")
        image = work_image.resize((self.size, self.size), resample=Image.BILINEAR)
        self.url = self.save_avatar(image)
        if AVATAR_HIGH_RESOLUTION:
            high_res_image = work_image.resize((self.size * 2, self.size * 2), resample=Image.BILINEAR)
            self.high_res_url = self.save_avatar(high_res_image, high_res=True).decode('utf-8')
        return self.url

    def save_avatar(self, image, high_res=False):
        image_io = BytesIO()
        image.save(image_io, format=self.content_type)
        image_io.seek(0, os.SEEK_END)
        image_io_length = image_io.tell()
        image_name = self.name(high_res)
        try:
            django_file = InMemoryUploadedFile(image_io, None, image_name, 'image/{0}'.format(self.content_type.lower()), image_io_length, None)
            AVATAR_STORAGE_BACKEND.save(self.path(high_res), django_file)
            return AVATAR_STORAGE_BACKEND.url(self.path(high_res))
        except Exception as e:
            raise e

    def get_avatar_url(self):
        """
            returns the url of the avatar on the storage backed
        """
        try:
            if has_gravatar(self.user.email):
                self.css_class = "gravatar"
                self.url = escape(get_gravatar_url(email=self.user.email, size=self.size))
                return self.url
        except NameError:
            pass

        self.css_class = "initial-avatar"
        if AVATAR_STORAGE_BACKEND.exists(self.path()):
            self.url = AVATAR_STORAGE_BACKEND.url(self.path())
        else:
            self.url = self.genavatar()
        if AVATAR_HIGH_RESOLUTION:
            self.high_res_url = AVATAR_STORAGE_BACKEND.url(self.path(high_res=True))
            return self.url, self.high_res_url
        return self.url

    def get_avatar(self):
        """
            returns an html img tag with the avatar
        """
        self.get_avatar_url()
        if AVATAR_HIGH_RESOLUTION:
            return '<img class="{css_class}" src="{src}" srcset="{src} 1x, {high_res_src} 2x" width="{width}" height="{height}"/>'.format(
                css_class=self.css_class,
                src=self.url,
                high_res_src=escape(self.high_res_url),
                width=self.size,
                height=self.size
            )
        return '<img class="{css_class}" src="{src}" width="{width}" height="{height}"/>'.format(
            css_class=self.css_class,
            src=self.url,
            width=self.size,
            height=self.size
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.contrib.auth.models import User
from django.utils.safestring import mark_safe
from django import template
from ..generator import GRAVATAR_DEFAULT_SIZE, AVATAR_SHAPE
from ..utils import get_avatar_backend

register = template.Library()


@register.simple_tag(name='get_initial_avatar')
def get_initial_avatar(user_or_email, size=GRAVATAR_DEFAULT_SIZE, shape=AVATAR_SHAPE):
    """ Builds an avatar <img> tag from an user or email """

    if hasattr(user_or_email, 'email'):
        user = user_or_email
        email = user.email
    else:
        email = user_or_email
        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            return mark_safe('<img src="" width="{width}" height="{height}"/>'.format(width=size, height=size))

    avatar_backend = get_avatar_backend()
    return mark_safe(avatar_backend(user, size=int(size), shape=shape).get_avatar())


from django.conf import settings
from importlib import import_module


class AvatarShapeException(BaseException):
    pass


class AvatarForegroundColorException(BaseException):
    pass


def get_avatar_backend():
    backend_name = getattr(settings, 'AVATAR_GENERATOR_BACKEND', 'initial_avatars.generator.AvatarGenerator')
    backend_module_name, backend_cls_name = backend_name.rsplit('.', 1)
    backend_module = import_module(backend_module_name)
    backend_class = getattr(backend_module, backend_cls_name)
    return backend_class

AVATAR_SHAPE_SETTINGS = {
    'circle': {
        'image_format': 'png',
        'content_type': 'PNG',
        'color_channels': 'RGBA'
    },
    'square': {
        'image_format': 'jpg',
        'content_type': 'JPEG',
        'color_channels': 'RGB'
    }
}

AVATAR_FOREGROUND_COLORS = {
    'white': (255, 255, 255),
    'black': (0, 0, 0)
}



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-28 11:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django


class Migration(migrations.Migration):

    dependencies = [
        ('initial_avatars', '0001_initial'),
    ]

    if django.VERSION < (2, 0, 0):
        operations = [
            migrations.AlterField(
                model_name='Background',
                name='user',
                field=models.OneToOneField(
                    related_name='initial_avatars',
                    primary_key=True,
                    serialize=False,
                    to=settings.AUTH_USER_MODEL,
                    on_delete=models.CASCADE
                )
            ),
        ]




import os
from setuptools import setup
from initial_avatars import __version__

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django4-initial-avatars',
    version=__version__,
    packages=['initial_avatars', 'initial_avatars.templatetags'],
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to get avatars based on username and initials if no gravatars is associated with the email address. Forked from axiome-oss.',
    keywords='django avatar gravatar',
    long_description=open('README.rst').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Urolas/django4-initial-avatars',
    author='Sophanna NGOV',
    author_email='sophanna.ngov@gmail.com',
    install_requires=[
        'Pillow',
        'Django',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 4.2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    extras_require={
        'gravatar': ['django-gravatar2>=1.3.0'],
    },
    zip_safe=False,
    test_suite="runtests.runtests",
)
